//*****************************************************************************
//  Copyright (c) 2004, Antivari Software Design
//*****************************************************************************
//
//  Project name       :
//  Subsystem          :
//  Module file        :
//  Created by         :
//  Annotations        :
//
//
//*****************************************************************************
//  Description:
//
//
//
//*****************************************************************************
//  Changes:
//
//
//
//*****************************************************************************

#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>

#include <FileCtrl.hpp>
#include <StrUtils.hpp>
#include <IniFiles.hpp>

#include "main.h"

#pragma package(smart_init)
#pragma resource "*.dfm"

TServerMainForm *ServerMainForm;

__fastcall TServerMainForm::TServerMainForm(TComponent* Owner)
        : TForm(Owner)
{

   // Ucitaj ime servisa
   strcpy ( ServName, "TicketService" );

   // Obezbedi da je prvi Tab uvek General
   PageControl->ActivePage = ServiceTabSheet;

   // Procesiranje trenutno nije aktivno
   Querying = false;

   // Maskiraj 'Details...' taster
   DetailsButton->Visible = false;

   // get the current style flags
   LONG const dwStyle1 = GetWindowLong ( PortEdit->Handle, GWL_STYLE );
   LONG const dwStyle2 = GetWindowLong (  LogEdit->Handle, GWL_STYLE );

   // add ES_NUMBER to these
   SetWindowLong ( PortEdit->Handle, GWL_STYLE, dwStyle1 | ES_NUMBER );
   SetWindowLong (  LogEdit->Handle, GWL_STYLE, dwStyle2 | ES_NUMBER );

   // Ucitaj konfiguraciju
   LoadConfig();

   // Snimi konfiguraciju
   SaveConfig();


   // Sync
   bool IsChecked = LogCheckBox->Checked;

   LogEdit->Enabled    = IsChecked;
   LogKbLabel->Enabled = IsChecked;

   // Inicijalizuj SCM
   H1 = NULL;

   // Probaj da otvoris SCM
   TryToOpenSCManager();

   // Stanje servera
   StatusPanel->Caption = GetServiceStatus();
}

__fastcall TServerMainForm::~TServerMainForm()
{
   // Unisti H1 ako je alociran
   if ( H1 )
      CloseServiceHandle ( H1 );
}
void __fastcall TServerMainForm::StartButtonClick(TObject *Sender)
{
   START_TYPE = "1";
   SetIniValue( "Server", "StartType", START_TYPE );

   bool R = CommandStart();

   if ( !R )
   {
      HandleGetLastError();
      START_TYPE = "0";
      SetIniValue( "Server", "StartType", START_TYPE );
   }

   QueryStatus();
}

void __fastcall TServerMainForm::PauseButtonClick(TObject *Sender)
{
   START_TYPE = "0";
   SetIniValue( "Server", "StartType", START_TYPE );

   bool R = CommandPause();

   if ( !R )
   {
      HandleGetLastError();
      START_TYPE = "1";
      SetIniValue( "Server", "StartType", START_TYPE );
   }

   QueryStatus();
}

void __fastcall TServerMainForm::ContinueButtonClick(TObject *Sender)
{
   START_TYPE = "1";
   SetIniValue( "Server", "StartType", START_TYPE );

   bool R = CommandContinue();

   if ( !R )
   {
      HandleGetLastError();
      START_TYPE = "0";
      SetIniValue( "Server", "StartType", START_TYPE );
   }

   QueryStatus();
}

void __fastcall TServerMainForm::StopButtonClick(TObject *Sender)
{
   START_TYPE = "0";
   SetIniValue( "Server", "StartType", START_TYPE );

   bool R = CommandStop();

   if ( !R )
   {
      HandleGetLastError();
      START_TYPE = "1";
      SetIniValue( "Server", "StartType", START_TYPE );
   }

   QueryStatus();
}

void __fastcall TServerMainForm::RefreshButtonClick(TObject *Sender)
{
   QueryStatus();
}

void __fastcall TServerMainForm::DetailsButtonClick(TObject *Sender)
{
   MessageDlg( StatusPanel->Caption + "\n" + SCManagerErrMsg,
               mtInformation, TMsgDlgButtons() << mbOK, 0 );
}

void __fastcall TServerMainForm::PageControlChanging(TObject *Sender,
      bool &AllowChange)
{
   if ( Querying )
      AllowChange = false;
}

void __fastcall TServerMainForm::FormKeyPress(TObject *Sender, char &Key)
{
   if ( Key == VK_ESCAPE )
   {
      Key = 0;
      Close();
   }
}

void __fastcall TServerMainForm::FormActivate(TObject *Sender)
{
   CfgToDlg();
}

void __fastcall TServerMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
   Application->ProcessMessages();

   if ( Querying )
      return;
}

void __fastcall TServerMainForm::OkButtonClick(TObject *Sender)
{
   if ( Querying )
      return;

   Close();
}

void __fastcall TServerMainForm::PageControlChange(TObject *Sender)
{
   if ( PageControl->ActivePage == ServiceTabSheet )
      QueryStatus();
}


void TServerMainForm::QueryStatus()
{
   if ( !H1 )
      return;

   Querying = true;

   StatusPanel->Caption = "Provera statusa ...";

   SetButtonState( BS_NONE );

   Application->ProcessMessages();

   Sleep( 2000 );

   StatusPanel->Caption = GetServiceStatus();

   Application->ProcessMessages();

   RefreshButton->Enabled = true;

   Querying = false;

}

bool TServerMainForm::SetIniValue( AnsiString Section, AnsiString Key, AnsiString Value )
{
   bool Result = true;

   TIniFile *pIni = NULL;

   try
   {
      pIni = new TIniFile( ChangeFileExt( ParamStr(0), ".ini" ) );
      pIni->WriteString( Section, Key, Value );
   }
   catch ( const Exception &e )
   {
      Result = false;
   }

   if ( pIni )
      delete pIni;

   return ( Result );
}

bool TServerMainForm::CommandStart()
{
   bool Result;
   SC_HANDLE H2 = OpenService ( H1, ServName, SERVICE_ALL_ACCESS );
   Result = StartService ( H2, NULL, NULL );
   if ( !H2 ) CloseServiceHandle ( H2 );

   return ( Result );
}

bool TServerMainForm::CommandPause()
{
   bool Result;
   SC_HANDLE H2 = OpenService ( H1, ServName, SERVICE_ALL_ACCESS );
   Result = ControlService ( H2, SERVICE_CONTROL_PAUSE, &sstatus );
   if ( ! H2 ) CloseServiceHandle ( H2 );
   return ( Result );
}

bool TServerMainForm::CommandContinue()
{
   bool Result;
   SC_HANDLE H2 = OpenService ( H1, ServName, SERVICE_ALL_ACCESS );
   Result = ControlService ( H2, SERVICE_CONTROL_CONTINUE, &sstatus );
   if ( ! H2 ) CloseServiceHandle ( H2 );
   return ( Result );
}

bool TServerMainForm::CommandStop()
{
   bool Result;
   SC_HANDLE H2 = OpenService ( H1, ServName, SERVICE_ALL_ACCESS );
   Result = ControlService ( H2, SERVICE_CONTROL_STOP, &sstatus );
   if ( ! H2 ) CloseServiceHandle ( H2 );
   return ( Result );
}

bool TServerMainForm::CommandUserDefinedReload()
{
   bool Result;
   SC_HANDLE H2 = OpenService ( H1, ServName, SERVICE_USER_DEFINED_CONTROL );
   Result = ControlService ( H2, 200, &sstatus );
   if ( ! H2 ) CloseServiceHandle ( H2 );
   return ( Result );
}

bool TServerMainForm::CommandQueryServiceStatus()
{
   bool Result;
   SC_HANDLE H2 = OpenService ( H1, ServName, SERVICE_QUERY_STATUS );
   Result = QueryServiceStatus ( H2, &sstatus );
   if ( ! H2 ) CloseServiceHandle ( H2 );
   return ( Result );
}


void __fastcall TServerMainForm::SendButtonClick(TObject *Sender)
{
   bool R;
   if ( CommandQueryServiceStatus() == true )
   {
      if ( sstatus.dwCurrentState == SERVICE_RUNNING ||
           sstatus.dwCurrentState == SERVICE_PAUSED )
      {
         R = CommandUserDefinedReload();
         if ( !R ) HandleGetLastError();
      }
   }

   // Obezbedi da je prvi Tab uvek General
   PageControl->ActivePage = ServiceTabSheet;

   QueryStatus();

}

AnsiString TServerMainForm::GetServiceStatus()
{
   AnsiString ServStatus;

   bool Result = CommandQueryServiceStatus();

   if ( ! Result )
   {
      DWORD Err = ::GetLastError();
      switch ( Err )
      {
         case ERROR_ACCESS_DENIED:
            ServStatus = "Izabrani postupak nije zapoet sa odgovarajuim pravima pristupa.";
            break;
         case ERROR_INVALID_HANDLE:
            ServStatus = "Proverite da li je server instaliran.";
            break;
         default:
            ServStatus = "[ Nepoznata greka ]";
            break;
      }

      SetButtonState( BS_REFRESHONLY );

   }
   else
   {

      switch ( sstatus.dwCurrentState )
      {
         case SERVICE_STOPPED :
            ServStatus = "Server nije pokrenut.";
            SetButtonState( BS_STOPPED );
            break;

         case SERVICE_START_PENDING:
            ServStatus = "Server se pokree...";
            SetButtonState( BS_REFRESHONLY );
            break;

         case SERVICE_STOP_PENDING:
            ServStatus = "Server se zaustavlja...";
            SetButtonState( BS_REFRESHONLY );
            break;

         case SERVICE_RUNNING:
            ServStatus = "Server je pokrenut.";
            SetButtonState( BS_STARTED );
            break;

         case SERVICE_CONTINUE_PENDING:
            ServStatus = "Server se nastavlja...";
            SetButtonState( BS_REFRESHONLY );
            break;
         case SERVICE_PAUSE_PENDING:
            ServStatus = "Server se pauzira...";
            SetButtonState( BS_REFRESHONLY );
            break;
         case SERVICE_PAUSED:
            ServStatus = "Server je pauziran.";
            SetButtonState( BS_PAUSED );
            break;

         default:
            ServStatus = "[ Stanje servera je nepoznato ]";
            SetButtonState( BS_REFRESHONLY );
            break;
      }

   }

   return ( ServStatus );
}

void TServerMainForm::SetButtonState( BUTTON_STATE BS )
{
   switch ( BS )
   {
      case BS_STARTED:

         StartButton->Enabled    = false;
         PauseButton->Enabled    = true;
         ContinueButton->Enabled = false;
         StopButton->Enabled     = true;
         RefreshButton->Enabled  = true;
         SendButton->Enabled     = true;

         break;

      case BS_STOPPED:

         StartButton->Enabled    = true;
         PauseButton->Enabled    = false;
         ContinueButton->Enabled = false;
         StopButton->Enabled     = false;
         RefreshButton->Enabled  = true;
         SendButton->Enabled     = false;

         break;

      case BS_PAUSED:

         StartButton->Enabled    = false;
         PauseButton->Enabled    = false;
         ContinueButton->Enabled = true;
         StopButton->Enabled     = true;
         RefreshButton->Enabled  = true;
         SendButton->Enabled     = true;

         break;

      case BS_NONE:

         StartButton->Enabled    = false;
         PauseButton->Enabled    = false;
         ContinueButton->Enabled = false;
         StopButton->Enabled     = false;
         RefreshButton->Enabled  = false;
         SendButton->Enabled     = false;

         break;

      case BS_REFRESHONLY:

         StartButton->Enabled    = false;
         PauseButton->Enabled    = false;
         ContinueButton->Enabled = false;
         StopButton->Enabled     = false;
         RefreshButton->Enabled  = true;
         SendButton->Enabled     = false;

         break;
   }

}



void TServerMainForm::TryToOpenSCManager()
{

   H1 = OpenSCManager ( NULL, NULL, SC_MANAGER_ALL_ACCESS );

   if ( H1 == NULL )
   {
      SCMErr = ::GetLastError();

      StaticResources();

      DetailsButton->Visible = true;
      SetButtonState( BS_NONE );
   }

}

void TServerMainForm::StaticResources()
{
   switch ( SCMErr )
   {
      case ERROR_ACCESS_DENIED:
           SCManagerErrMsg = "Zahtevana vrsta pristupa je odbijena.";
           break;

      case ERROR_DATABASE_DOES_NOT_EXIST:
           SCManagerErrMsg = "Traena baza podataka ne postoji.";
           break;

      case ERROR_INVALID_PARAMETER:
           SCManagerErrMsg = "Parametari za pristup nisu ispravni.";
           break;

      default:
           if ( IsWindows9X() == true )
              SCManagerErrMsg = "Servisi nisu podrani od Windows 98 opretivnog sistema.";
           else
              SCManagerErrMsg = "[ Nepoznata greka ]";
           break;
   }

   StatusPanel->Caption = "Upravljanje servisom nije dostupno";

}

bool TServerMainForm::IsWindows9X()
{
    OSVERSIONINFO info;
    info.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
    GetVersionEx( &info );

    if ( info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
       return ( true );

    return ( false );
}


void TServerMainForm::HandleGetLastError()
{
   AnsiString ErrOccured = "Komanda nije prihvena";
   DWORD Err = ::GetLastError();

   switch ( Err )
   {

      case ERROR_ACCESS_DENIED:
           MessageDlg( ErrOccured + "\n" +
                       "Izabrani postupak nije zapoet\n"
                       "sa odgovarajuim pravima pristupa.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_DEPENDENT_SERVICES_RUNNING:
           MessageDlg( ErrOccured + "\n" +
                       "Servis se ne moe zaustaviti radi drugih\n"
                       "aktivnih servisa koji su zavisni o njega.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_INVALID_SERVICE_CONTROL:
           MessageDlg( ErrOccured + "\n" +
                       "Zatraeni kontrolni kod nije ispravan\n"
                       "i zato ga servis ne moe prihvatiti.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_CANNOT_ACCEPT_CTRL:
           MessageDlg( ErrOccured + "\n" +
                       "Zatraeni kontrolni kod ne moe se poslati servisu\n"
                       "jer je stanje servisa\n"
                       "<Zaustavljeno>, <Trenutno u kretanju> ili <Trenutno u zaustavljanju>.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
          break;

      case ERROR_SERVICE_NOT_ACTIVE:
           MessageDlg( ErrOccured + "\n" +
                       "Server nije pokrenut.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_REQUEST_TIMEOUT:
           MessageDlg( ErrOccured + "\n" +
                       "Servis nije odgovorio na zahtev za pokretanjem\n"
                       "u prihvatljivom vremenskom intervalu",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      // STARTING

      // case ERROR_ACCESS_DENIED:
      // "The specified handle was not opened with\nSERVICE_START access."
      // break;

      case ERROR_INVALID_HANDLE:
           MessageDlg( ErrOccured + "\n" +
                       "Izabrani postupak nije ispravan.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_PATH_NOT_FOUND:
           MessageDlg( ErrOccured + "\n" +
                       "Binarna datoteka servisa nije pronaena.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_ALREADY_RUNNING:
           MessageDlg( ErrOccured + "\n" +
                       "Jedna instanca servisa ve je pokrenuta.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_DATABASE_LOCKED:
           MessageDlg( ErrOccured + "\n" +
                       "Baza podataka je zakljuana.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_DEPENDENCY_DELETED:
           MessageDlg( ErrOccured + "\n" +
                       "Ovaj servis je ovisan o drugom servisu koji ne postoji\n"
                       "ili je obeleen za uklanjanje.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_DEPENDENCY_FAIL:
           MessageDlg( ErrOccured + "\n" +
                       "Ovaj servis je ovisan o drugom servisu\n"
                       "koji nije uspeo da se pokrene.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_DISABLED:
           MessageDlg( ErrOccured + "\n" +
                       "Servis je onemoguen.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_LOGON_FAILED:
           MessageDlg( ErrOccured + "\n" +
                       "Servis se ne moe prijaviti.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      case ERROR_SERVICE_MARKED_FOR_DELETE:
           MessageDlg( ErrOccured + "\n" +
                       "Servis je obeleen za uklanjanje.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );

           break;

      case ERROR_SERVICE_NO_THREAD:
           MessageDlg( ErrOccured + "\n" +
                       "Programska nit se ne moe kreirati za Win32 servis.",
                       mtInformation, TMsgDlgButtons() << mbOK, 0 );
           break;

      //case ERROR_SERVICE_REQUEST_TIMEOUT:
      //     MBox ( ErrOccured.c_str(), X(IDS_SER_MAIN_GLE_).c_str() );
      // "The service did not respond to the start request\nin a timely fashion."
      //     break;
   }
}


void __fastcall TServerMainForm::LogCheckBoxClick(TObject *Sender)
{
   bool IsChecked = LogCheckBox->Checked;

   LogEdit->Enabled    = IsChecked;
   LogKbLabel->Enabled = IsChecked;

   if ( IsChecked )
      LIMITING = "1";
   else
      LIMITING = "0";

   SetIniValue( "Log", "Limiting", LIMITING );

}


void TServerMainForm::LoadConfig()
{
   TIniFile *pIni = NULL;

   try
   {
      pIni = new TIniFile( ChangeFileExt( ParamStr(0), ".ini" ) );

      LIMITING       = pIni->ReadString( "Log", "Limiting", "1" );
      LINES          = pIni->ReadString( "Log", "Lines", "500" );
      PORT           = pIni->ReadString( "Connections", "Port", "11059" );
      START_TYPE     = pIni->ReadString( "Server", "StartType", "0" );

      if ( (LIMITING != "1") && (LIMITING != "0") )
      {
         LIMITING = "1";
      }

      if ( (START_TYPE != "1") && (START_TYPE != "0") )
      {
         START_TYPE = "0";
      }

      try
      {
         int lines = StrToInt( LINES );

         if ( lines < 1 || lines > 32768 )
            LINES = "500";

      }
      catch ( const Exception &e )
      {
         LINES = "500";
      }


      try
      {
         int port = StrToInt( PORT );

         if ( port < 1 || port > 32768 )
            PORT = "11059";

      }
      catch ( const Exception &e )
      {
         PORT = "11059";
      }

   }
   catch ( const Exception &e )
   {
      // ...
   }

   if ( pIni )
      delete pIni;

}


void TServerMainForm::SaveConfig()
{
   TIniFile *pIni = NULL;

   try
   {
      pIni = new TIniFile( ChangeFileExt( ParamStr(0), ".ini" ) );

      pIni->WriteString( "Log", "Limiting", LIMITING );
      pIni->WriteString( "Log", "Lines", LINES );
      pIni->WriteString( "Connections", "Port", PORT );
      pIni->WriteString( "Server", "StartType", START_TYPE );

   }
   catch ( const Exception &e )
   {
      // ...
   }

   if ( pIni )
      delete pIni;

}

void TServerMainForm::CfgToDlg()
{
   if ( LIMITING == "1" )
      LogCheckBox->Checked = true;
   else
      LogCheckBox->Checked = false;

   LogEdit->Text = LINES;
   PortEdit->Text = PORT;

}



void __fastcall TServerMainForm::PortEditChange(TObject *Sender)
{
   PORT = PortEdit->Text;

   try
   {
      int port = StrToInt( PORT );

      if ( port < 1 || port > 32768 )
         PORT = "11059";

   }
   catch ( const Exception &e )
   {
      PORT = "11059";
   }

   SetIniValue( "Connections", "Port", PORT );

   PortEdit->Text = PORT;
}

void __fastcall TServerMainForm::LogEditChange(TObject *Sender)
{
   LINES = LogEdit->Text;

   try
   {
      int lines = StrToInt( LINES );

      if ( lines < 1 || lines > 32768 )
         LINES = "500";

   }
   catch ( const Exception &e )
   {
      LINES = "500";
   }

   SetIniValue( "Log", "Lines", LINES );

   LogEdit->Text = LINES;
}



